#!/usr/bin/env python
"""

Create a sioseis script for plotting data in a segy file.  The
intention is that the resulting script will be tweaked to look nice.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segysql-sioseis-plt.py,v 1.3 2005/09/25 19:57:01 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.3 $' # See man ident
__date__ = '$Date: 2005/09/25 19:57:01 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'

import sqlite
import os
import sys

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from optparse import OptionParser

vcvs = "$Revision: 1.3 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options]",
                      version="%prog "+VERSION+' - '+version_cvs)
myparser.add_option('-d','--database',dest='dbfilename',default='segy.db',
                    help='Database containing imported segy file and trace header info [default: %default]')
myparser.add_option('-f','--file-key',dest='fileKey',type='int',default='1',
                    help='Which file from the database to use for the plot [default: %default]')

myparser.add_option('-m','--min-shot',dest='minShot',type='int',default=None,
                    help='Starting shot to plot.  Finds the first shot if not specified [default: %default]')
myparser.add_option('-M','--max-shot',dest='maxShot',type='int',default=None,
                    help='Starting shot to plot.  Finds the first shot if not specified [default: %default]')

myparser.add_option('-s','--shot-key',dest='shotKey',type='string',default='FieldRecNo',
                    help='Database key to use for shots.  E.g. Shotpoint or EnsembleNo [default: %default]'+
                    "Use 'sqlite segy.db \".schema segyTrace\"' to see options")


myparser.add_option('-S','--sioseis',dest='sioseis',type='string',default='sioseis',
                    help='Sioseis binary to use [default: %default]')

myparser.add_option('--display',dest='display',type='string',default=None,
                    help='Program to use to view the results.  e.g. xv or display [default: %default]')

myparser.add_option('-o','--outfile',dest='outfile',type='string',default='plot.bash',
                    help='File to write the script to [default: %default]')

myparser.add_option('-b','--basename',dest='basename',type='string',default=None,
                    help='Plotting files base name images.  Creates one if None. [default: %default]')


#myparser.add_option('--wbt3',dest='wbt3',default=False, action='store_true',
#                    help='Turn on water bottom picking')


myparser.add_option('-v',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')


if __name__ == "__main__":
    (options,args) = myparser.parse_args()

    if options.verbosity >= TRACE:     print "Verbosity = ", options.verbosity
    if options.verbosity >= 2*BOMBASTIC: print "options = ", options


    if not os.access(options.dbfilename,os.R_OK):
        print "ERROR: database does not exist...",options.dbfilename
        sys.exit("  No shoes, no shirt, no service")
    cx = sqlite.connect(options.dbfilename)

    cu = cx.cursor()
    cu.execute("SELECT filename FROM segyFile WHERE filekey="+str(options.fileKey)+";")
    row = cu.fetchone()
    try:
        filename = row['filename']
    except TypeError:
        sys.exit("ERROR: no file with this key in the database - "+str(options.fileKey))

    #
    # Lookup the shot range
    #
    if None == options.minShot and None == options.maxShot:
        # Do both at the same time for speed
        searchStr =  "SELECT min("+options.shotKey+") AS min, max("+options.shotKey+") AS max "
        searchStr += " FROM segyTrace WHERE fileKey="+str(options.fileKey)
        if options.verbosity>=TRACE: print "Combining searches for speed"
        if options.verbosity>=VERBOSE: print searchStr
        cu.execute(searchStr)
        row = cu.fetchone()
        options.minShot = int(row['min'])
        options.maxShot = int(row['max'])

    if None == options.minShot:
        searchStr = "SELECT min("+options.shotKey+") AS min FROM segyTrace WHERE fileKey="+str(options.fileKey)
        if options.verbosity>=VERBOSE: print searchStr
        cu.execute(searchStr)
        options.minShot = int(cu.fetchone()['min'])
    if None == options.maxShot:
        searchStr = "SELECT max("+options.shotKey+") AS max FROM segyTrace WHERE fileKey="+str(options.fileKey)
        if options.verbosity>=VERBOSE: print searchStr
        cu.execute(searchStr)
        options.maxShot = int(cu.fetchone()['max'])

    if options.verbosity>=VERBOSE: print "shot range:  ",options.minShot,options.maxShot
    assert(options.minShot>=1)
    assert(options.minShot<options.maxShot)

    if None == options.basename:
        # FIX: strip off .sgy or .segy
        options.basename = filename + '-' + ("%05d"%options.minShot)+'-'+("%05d" % options.maxShot)
        if options.verbosity>=TERSE: print "basename: ",options.basename


    #
    # FIGURE THE BEST STARTING POINT IN TWTT for plotting
    #
    searchStr  = "SELECT min(Delay) AS min FROM segyTrace"
    searchStr += " WHERE fileKey="+str(options.fileKey)
    searchStr += " AND "+options.shotKey+">="+str(options.minShot)
    searchStr += " AND "+options.shotKey+"<="+str(options.maxShot)
    if options.verbosity >= VERBOSE: print searchStr
    cu.execute(searchStr)
    stime = float(cu.fetchone()['min'])/1000.
    if options.verbosity >= TRACE: print 'stime: ',stime


    searchStr  = "SELECT max(Delay) AS max FROM segyTrace"
    searchStr += " WHERE fileKey="+str(options.fileKey)
    searchStr += " AND "+options.shotKey+">="+str(options.minShot)
    searchStr += " AND "+options.shotKey+"<="+str(options.maxShot)
    if options.verbosity >= VERBOSE: print searchStr
    cu.execute(searchStr)
    maxDelay = float(cu.fetchone()['max'])/1000.
    if options.verbosity >= TRACE: print 'max delay: ',stime

    searchStr  = "SELECT min(SampleInterval) AS minInterval, max(SampleInterval) AS maxInterval "
    searchStr += ", min(TraceSamples) AS minSamples, max(TraceSamples) AS maxSamples "
    searchStr += " FROM segyTrace"
    searchStr += " WHERE fileKey="+str(options.fileKey)
    searchStr += " AND "+options.shotKey+">="+str(options.minShot)
    searchStr += " AND "+options.shotKey+"<="+str(options.maxShot)
    cu.execute(searchStr)
    row = cu.fetchone()
    maxInterval = float(row['maxInterval'])
    minInterval = float(row['minInterval'])

    maxSamples = float(row['maxSamples'])
    minSamples = float(row['minSamples'])

    if (maxInterval != minInterval):
        sys.exit ("\n ERROR: What do you expect me to do if the sample interval changes?\n  Bite me. I quit.")
    #print minSamples, maxSamples
    #print minInterval, maxInterval, minInterval/1000000., maxInterval/1000000. * maxSamples
    nsecs = maxInterval/1000000. * maxSamples


    ######################################################################
    # Write out the file!
    ######################################################################

    out = open(options.outfile,'w')
    if os.access("/bin/bash",os.X_OK):
        out.write('#!/bin/bash\n')
    else:
        out.write('#!/usr/bin/env bash\n')
    out.write('# script written by '+sys.argv[0]+'\n')
    out.write('\n')
    out.write('# stime  - time offset from shot to start plotting\n')
    out.write('# nsecs  - number of seconds of data to plot\n')
    out.write('# vscale - vertical scale in inches per sec\n')
    out.write('# trpin  - traces per inch on the horizontal\n')
    out.write('# wiggle - with 0 the it plots too much!\n')
    out.write('# opath  - where to write the file on disk\n')
    out.write('\n')
    out.write('# SEE ALSO: http://sioseis.ucsd.edu/plot.html\n')
    out.write('\n')

    procs = ['diskin','prout']
    #if (options.wbt3): procs.append('wbt3')
    procs.append('plot')
    if options.verbosity>=TRACE: print 'procs: ',procs

    procStr='';
    for proc in procs: procStr += proc + ' '
    print procStr

    out.write(options.sioseis+'<< EOF\n')
    out.write('\n')
    out.write('procs \n')
    out.write(procStr)
    out.write(' end\n')

    out.write('\n')
    out.write('diskin\n')
    out.write('  ipath '+filename+' end\n')
    out.write('end\n')

    # Print out what we are plotting  FIX: make prout optional
    numberInc = int((options.maxShot-options.minShot)/15)
    out.write('\n')
    out.write('prout\n')
    out.write('  fno '+str(options.minShot)+' lno '+str(options.maxShot)+' noinc '+str(numberInc)+' end\n')
    out.write('end\n')

# This is going to need more work.
#     if (options.wbt3):
#         out.write('\n');
#         out.write('wbt3\n');
#         out.write('  thres 0.0000000001 track 0.05 end\n');
#         out.write('end\n');
#         out.write('\n');

    # FIX: what is the proper page hight?
    vscale = int(float(7. / nsecs))
    print vscale
    trpin = 100

    out.write('plot\n')

    out.write('  fno  '+str(options.minShot) +'\n')
    out.write('  lno  '+str(options.maxShot) +'\n')

    out.write('  stime  '+str(stime) +'\n')
    out.write('  nsecs  '+str(nsecs) +'\n')
    out.write('  vscale '+str(vscale)+'\n')
    out.write('  trpin  '+str(trpin) +'\n')
    out.write('  wiggle 0\n')
    out.write('  def .1\n')
    out.write('  clip .01\n')
    out.write('  opath '+options.basename+'.sio end\n')
    out.write('end \n')

    out.write('\n')

    out.write('end\n')
    out.write('EOF\n')


    # Post processing
    out.write('\n')
    out.write('sio2sun '+options.basename+'.sio '+options.basename+'.sun\n')
    out.write('convert -rotate 90 '+options.basename+'.sun '+options.basename+'.png\n')
    if options.display != None:
        out.write(options.display+' '+options.basename+'.png\n')
