#!/usr/bin/env python

"""
Command utility to examine a SEGY file

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segystats.py,v 1.1 2005/09/25 22:03:22 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.1 $' # See man ident
__date__ = '$Date: 2005/09/25 22:03:22 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'

import os
import sys

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from segy import Segy
import segy

from optparse import OptionParser

# What version do we want to track?  This would be for cvs of this file
vcvs = "$Revision: 1.1 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)

myparser.add_option('-d','--driver',dest="drv",
                    help="specify which driver to parse data file.  [default: %default]",
                    default="segy_drv_rev1")
myparser.add_option('-f','--file',dest="file",action="append", type="string",
                    help="specify segy file to examine.  Specify one or more of these")

myparser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')
myparser.add_option('--noisy',dest='verbosity', action='store_const', const=2*BOMBASTIC,
                    help='Go for the max verbosity')


if __name__ == '__main__':
    (options,args) = myparser.parse_args()
    v = options.verbosity
    if v >= VERBOSE:   print '# Option values: ', options
    if v >= BOMBASTIC: print '# Arg    values: ', args

    for file in options.file:
        print
        print '# FILE: ',file
        s = Segy(file,drivername=options.drv,verbosity=options.verbosity)
        minSample =  10e99
        maxSample = -10e99
        for i in range(1,s.getNumberOfTraces()+1):
            data = s.getTraceData(i)
            minLocal = min(data)
            maxLocal = max(data)
            if (minLocal<minSample): minSample = minLocal
            if (maxLocal>maxSample): maxSample = maxLocal
            print i,minLocal,maxLocal,minSample,maxSample

# FIX: what else in terms of statistics would be good?
